use crate::vm::lib::magic::{COMPILER_VERSION_STRING, DEIMOS_MAGIC_STRING};
use crate::vm::lib::DmLib;

macro_rules! push_byte_array {
    ( $dest: expr, $arr: expr ) => {
        for b in $arr {
            $dest.push(b);
        }
    };
}

macro_rules! push_number_le {
    ( $dest: expr, $num: expr ) => {
        for b in $num.to_le_bytes() {
            $dest.push(b);
        }
    };
}

macro_rules! push_string {
    ( $dest: expr, $s: expr ) => {
        for b in $s.len().to_le_bytes() {
            $dest.push(b);
        }
        for b in $s.bytes() {
            $dest.push(b);
        }
    };
}

pub fn write_module(module: DmLib) -> Vec<u8> {
    // Push magic number
    let mut result: Vec<u8> = Vec::new();
    push_byte_array!(result, DEIMOS_MAGIC_STRING);

    // Push version string
    push_string!(result, COMPILER_VERSION_STRING);

    result
}
